@@ -16,7 +16,8 @@
  * \brief Calculate the output width or height, but not both.
  *
  * \param jpeg_width,jpeg_height dimensions of the input image
+ * \param switch_x_y whether to switch x and y dimensions, this is the case when the stored pixels differ from the displayed pixels due to a rotation
  */
-void aspect_ratio(const int jpeg_width, const int jpeg_height);
+void aspect_ratio(const int jpeg_width, const int jpeg_height, const int switch_x_y);
 
 #endif
