@@ -7,13 +7,16 @@
 #include "options.h"
 #include "round.h"
 
-void aspect_ratio(const int jpeg_width, const int jpeg_height) {
+void aspect_ratio(const int jpeg_width, const int jpeg_height, const int switch_x_y) {
 
 	// the 2.0f and 0.5f factors are used for text displays that (usually) have characters
 	// that are taller than they are wide.
 
-	#define CALC_WIDTH ROUND(2.0f * (float) height * (float) jpeg_width / (float) jpeg_height)
-	#define CALC_HEIGHT ROUND(0.5f * (float) width * (float) jpeg_height / (float) jpeg_width)
+	const int stored_width = switch_x_y ? jpeg_height : jpeg_width;
+	const int stored_height = switch_x_y ? jpeg_width : jpeg_height;
+
+	#define CALC_WIDTH ROUND(2.0f * (float) height * (float) stored_width / (float) stored_height)
+	#define CALC_HEIGHT ROUND(0.5f * (float) width * (float) stored_height / (float) stored_width)
 
 	// calc width
 	if ( auto_width && !auto_height ) {
@@ -22,7 +25,7 @@ void aspect_ratio(const int jpeg_width, const int jpeg_height) {
 		// adjust for too small dimensions	
 		while ( width==0 ) {
 			++height;
-			aspect_ratio(jpeg_width, jpeg_height);
+			aspect_ratio(stored_width, stored_height, 0);
 		}
 		
 		while ( termfit==TERM_FIT_AUTO && (width + use_border*2)>term_width ) {
@@ -30,7 +33,7 @@ void aspect_ratio(const int jpeg_width, const int jpeg_height) {
 			height = 0;
 			auto_height = 1;
 			auto_width = 0;
-			aspect_ratio(jpeg_width, jpeg_height);
+			aspect_ratio(stored_width, stored_height, 0);
 		}
 
 	}
@@ -41,7 +44,7 @@ void aspect_ratio(const int jpeg_width, const int jpeg_height) {
 
 		// adjust for too small dimensions
 		if ( height==0 ) {
-			height = ( jpeg_height == 1 )? 0 : 1;
+			height = ( stored_height == 1 )? 0 : 1;
 		}
 	}
 }
