@@ -1,6 +1,6 @@
 /*
  * Copyright 2006-2016 Christian Stigen Larsen
- * Copyright 2020 Christoph Raitzig
+ * Copyright 2020-2024 Christoph Raitzig
  * Distributed under the GNU General Public License (GPL) v2.
  */
 
@@ -17,8 +17,13 @@
 #include <string.h>
 #endif
 
+#include <stdint.h>
+
 #include "jpeglib.h"
 #include "png.h"
+#include "webp/decode.h"
+#include <libexif/exif-data.h>
+#include <libexif/exif-loader.h>
 #include <setjmp.h>
 
 #include "aspect_ratio.h"
@@ -26,9 +31,57 @@
 #include "jp2a.h"
 #include "options.h"
 #include "html.h"
+#include <math.h>
 
 #define ROUND(x) (int) ( 0.5f + x )
 
+static char DIRECTIONAL_CHARS[4] = "=/|\\";
+
+void print_margin_top(const Image *image, FILE *f) {
+	if ( centery && !( html || xhtml ) ) {
+		int total_margin = term_height - image->height;
+		if ( use_border ) {
+			total_margin -= 2;
+		}
+		if ( total_margin <= 0 )
+			return;
+		int margin_top = (int) floor(total_margin / 2);
+		for ( size_t i = 0; i < margin_top; i++ ) {
+			fprintf(f, "\n");
+		}
+	}
+}
+
+void print_margin_bottom(const Image *image, FILE *f) {
+	if ( centery && !( html || xhtml ) ) {
+		int total_margin = term_height - image->height;
+		if ( use_border ) {
+			total_margin -= 2;
+		}
+		if ( total_margin <= 0 )
+			return;
+		int margin_bottom = (int) ceil(total_margin / 2);
+		for ( size_t i = 0; i < margin_bottom; i++ ) {
+			fprintf(f, "\n");
+		}
+	}
+}
+
+void print_margin_start(const Image *image, FILE *f) {
+	if ( centerx && !( html || xhtml ) ) {
+		int total_margin = term_width - image->width;
+		if ( use_border ) {
+			total_margin -= 2;
+		}
+		if ( total_margin <= 0 )
+			return;
+		int margin_start = (int) floor(total_margin / 2);
+		for ( size_t i = 0; i < margin_start; i++ ) {
+			fprintf(f, " ");
+		}
+	}
+}
+
 void print_border(const int width) {
 	#ifndef HAVE_MEMSET
 	int n;
@@ -70,60 +123,142 @@ void print_image(Image *image, FILE *f) {
 
 	if ( html && !html_rawoutput ) print_html_image_start(f);
 	else if ( xhtml && !html_rawoutput ) print_xhtml_image_start(f);
-	if ( use_border ) print_border(image->width);
+	print_margin_top(image, f);
+	if ( use_border ) {
+		print_margin_start(image, f);
+		print_border(image->width);
+	}
 
 	(!usecolors? print_image_no_colors : print_image_colors) (image, ascii_palette_length - 1, f);
 
-	if ( use_border ) print_border(image->width);
+	if ( use_border ) {
+		print_margin_start(image, f);
+		print_border(image->width);
+	}
+	print_margin_bottom(image, f);
 	if ( html && !html_rawoutput ) print_html_image_end(f);
 	else if ( xhtml && !html_rawoutput ) print_xhtml_image_end(f);
 }
 
+int get_pixel_index(const Image* const image, const int x, const int y) {
+	float fx;
+	float fy;
+	switch (image->orientation) {
+		case HORIZONTAL:
+		case MIRROR_HORIZONTAL_ROTATE_90:
+			fx = flipx ? image->width - x - 1 : x;
+			fy = flipy ? image->height - y - 1 : y;
+			break;
+		case MIRROR_HORIZONTAL:
+		case ROTATE_270:
+			fx = flipx ? x : image->width - x - 1;
+			fy = flipy ? image->height - y - 1 : y;
+			break;
+		case ROTATE_180:
+		case MIRROR_HORIZONTAL_ROTATE_270:
+			fx = flipx ? x : image->width - x - 1;
+			fy = flipy ? y : image->height - y - 1;
+			break;
+		case MIRROR_VERTICAL:
+		case ROTATE_90:
+			fx = flipx ? image->width - x - 1 : x;
+			fy = flipy ? y : image->height - y - 1;
+			break;
+
+	}
+	const int cx = fx < 0 ? 0 : fx > image->width - 1 ? image->width - 1 : fx;
+	const int cy = fy < 0 ? 0 : fy > image->height - 1 ? image->height - 1 : fy;
+	if ( image->switch_x_y ) {
+		return cy + cx * image->src_width;
+	}
+	return cx + cy * image->src_width;
+}
+
+typedef struct {
+	float x;
+	float y;
+} vec2;
+
+vec2 get_image_gradient(const Image* const image, const int x, const int y) {
+	const float kernel_x[4] = {
+		-1., 0.,
+		 1., 0.
+	};
+	const float kernel_y[4] = {
+		-1., 1.,
+		 0., 0.
+	};
+	const float patch[4] = {
+		image->pixel[get_pixel_index(image, x  , y  )],
+		image->pixel[get_pixel_index(image, x+1, y  )],
+		image->pixel[get_pixel_index(image, x  , y+1)],
+		image->pixel[get_pixel_index(image, x+1, y+1)],
+	};
+	vec2 grad = {0., 0.};
+	for( int i = 0; i < 4; ++i ) {
+		grad.x += kernel_x[i] * patch[i];
+		grad.y += kernel_y[i] * patch[i];
+	}
+	return grad;
+}
+
+float magnitude( const vec2 v ) {
+	return sqrtf( v.x*v.x + v.y*v.y );
+}
+
+float direction( const vec2 v ) {
+	return atan(v.y / v.x);
+}
+
 void print_image_colors(const Image* const image, const int chars, FILE* f) {
 
-	int x, y;
-	int xstart, xend, xincr;
+	for ( int y=0;  y < image->height; ++y ) {
 
-	for ( y=0;  y < image->height; ++y ) {
+		print_margin_start(image, f);
 
 		if ( use_border ) fprintf(f, "|");
 
-		xstart = 0;
-		xend   = image->width;
-		xincr  = 1;
+		for ( int x=0; x < image->width; x += 1 ) {
 
-		if ( flipx ) {
-			xstart = image->width - 1;
-			xend = -1;
-			xincr = -1;
-		}
-
-		for ( x=xstart; x != xend; x += xincr ) {
-
-			float Y = image->pixel[x + (flipy? image->height - y - 1 : y ) * image->width];
-			float Y_inv = 1.0f - Y;
-			float R = image->red  [x + (flipy? image->height - y - 1 : y ) * image->width];
-			float G = image->green[x + (flipy? image->height - y - 1 : y ) * image->width];
-			float B = image->blue [x + (flipy? image->height - y - 1 : y ) * image->width];
-			float A = image->alpha [x + (flipy? image->height - y - 1 : y ) * image->width];
+			const int pixel_index = get_pixel_index(image, x, y);
+			float Y = image->pixel[pixel_index];
+			float R = image->red  [pixel_index];
+			float G = image->green[pixel_index];
+			float B = image->blue [pixel_index];
+			float A = image->alpha[pixel_index];
 			R *= A;
 			G *= A;
 			B *= A;
+			const vec2 gradient = get_image_gradient(image, x, y);
+			int pos = ROUND((float)chars * Y);
 
-			const int pos = ROUND((float)chars * (!invert? Y_inv : Y));
-			int i = ROUND((float)pos * A);
-#if ASCII
-			char ch = ascii_palette[i];
-#define	PRINTF_FORMAT_TYPE "%c"
-#else
-			char ch[MB_LEN_MAX + 1];
-			ch[0] = ascii_palette[ascii_palette_indizes[i]];
-			for ( size_t j = 1; j < ascii_palette_lengths[i]; j++ ) {
-				ch[j] = ascii_palette[ascii_palette_indizes[i] + j];
+			if( edges_only && magnitude(gradient) < edge_threshold ) {
+				pos = 0;
 			}
-			ch[ascii_palette_lengths[i]] = '\0';
+
+			int i = invert? pos : chars - pos;
+			i = ROUND((float)i * A);
+
+			char ch[MB_LEN_MAX + 1];
 #define PRINTF_FORMAT_TYPE "%s"
+
+#if ASCII
+			char* char_start = &ascii_palette[i];
+			size_t char_len = 1;
+#else
+			char* char_start = &ascii_palette[ascii_palette_indizes[i]];
+			size_t char_len = ascii_palette_lengths[i];
 #endif
+			if( magnitude(gradient) > edge_threshold ) {
+				// scale the gradient direction in the range -2 to 2, then add .5 to offset direction bins to match character directions
+				float direction_scaled = direction(gradient) / M_PI * 4. + .5;
+				// use +4 and fmod to bring the direction into the range 0-4, then use (int) to get an index 0-3 into DIRECTIONAL_CHARS array
+				char_start = &DIRECTIONAL_CHARS[ (int) fmod(direction_scaled + 4., 4.) ];
+				char_len = 1;
+			}
+
+			memcpy(ch, char_start, char_len);
+			ch[char_len] = '\0';
 
 			const float min = 1.0f / 255.0f;
 
@@ -268,8 +403,6 @@ void print_image_colors(const Image* const image, const int chars, FILE* f) {
 }
 
 void print_image_no_colors(const Image* const image, const int chars, FILE *f) {
-	int x, y;
-
 #if ASCII
 	#ifdef WIN32
 	char *line = (char*) malloc(image->width + 1);
@@ -283,62 +416,62 @@ void print_image_no_colors(const Image* const image, const int chars, FILE *f) {
 	#else
 	char line[image->width * MB_LEN_MAX + 1];
 	#endif
-	int curLinePos;
 	line[image->width * MB_LEN_MAX] = 0;
 #endif
 
-	for ( y=0; y < image->height; ++y ) {
+	for ( int y=0; y < image->height; ++y ) {
+		print_margin_start(image, f);
 
-#if ! ASCII
-		curLinePos = flipx? image->width * MB_LEN_MAX : 0;
-#endif
-		for ( x=0; x < image->width; ++x ) {
+		int curLinePos = 0;
+		for ( int x=0; x < image->width; ++x ) {
 
-			const float lum = image->pixel[x + (flipy? image->height - y - 1 : y) * image->width];
-			const float opacity = image->alpha[x + (flipy? image->height - y - 1 : y) * image->width];
-			const int pos = ROUND((float)chars * lum);
+			const int pixel_index = get_pixel_index(image, x, y);
+			const float lum = image->pixel[pixel_index];
+			const float opacity = image->alpha[pixel_index];
+			const vec2 gradient = get_image_gradient(image, x, y);
+			int pos = ROUND((float)chars * lum);
+
+			if( edges_only && magnitude(gradient) < edge_threshold ) {
+				pos = 0;
+			}
 
 			int i = invert? pos : chars - pos;
 			i = ROUND((float)i * opacity);
+
+			char* char_dest = &line[curLinePos];
 #if ASCII
-			line[flipx? image->width - x - 1 : x] = ascii_palette[i];
+			char* char_start = &ascii_palette[i];
+			size_t char_len = 1;
 #else
-			int paletteI = ascii_palette_indizes[i];
-			if ( flipx )
-				curLinePos -= ascii_palette_lengths[i];
-			line[curLinePos++] = ascii_palette[paletteI];
-			// Add as many bytes as the char's length
-			for ( size_t j = 1; j < ascii_palette_lengths[i]; j++ ) {
-				line[curLinePos++] = ascii_palette[++paletteI];
+			char* char_start = &ascii_palette[ascii_palette_indizes[i]];
+			size_t char_len = ascii_palette_lengths[i];
+#endif
+			if( magnitude(gradient) > edge_threshold ) {
+				// scale the gradient direction in the range -2 to 2, then add .5 to offset direction bins to match character directions
+				float direction_scaled = direction(gradient) / M_PI * 4. + .5;
+				// use +4 and fmod to bring the direction into the range 0-4, then use (int) to get an index 0-3 into DIRECTIONAL_CHARS array
+				char_start = &DIRECTIONAL_CHARS[ (int) fmod(direction_scaled + 4., 4.) ];
+				char_len = 1;
 			}
-			if ( flipx )
-				curLinePos -= ascii_palette_lengths[i];
-#endif
-		}
-#if ASCII
-		fprintf(f, !use_border? "%s\n" : "|%s|\n", line);
-#else
-		if ( !flipx ) {
+			memcpy(char_dest, char_start, char_len);
+			curLinePos += char_len;
 			line[curLinePos] = '\0';
-			fprintf(f, !use_border? "%s\n" : "|%s|\n", line);
-		} else {
-			fprintf(f, !use_border? "%s\n" : "|%s|\n", line + curLinePos);
 		}
-#endif
+		fprintf(f, !use_border? "%s\n" : "|%s|\n", line);
 	}
 
-	#ifdef WIN32
+#ifdef WIN32
 	free(line);
-	#endif
+#endif
 }
 
 void clear(Image* i) {
-	memset(i->yadds, 0, i->height * sizeof(int) );
+	memset(i->yadds, 0, i->src_height * sizeof(int) );
 	memset(i->pixel, 0, i->width * i->height * sizeof(float));
 	for ( int j = 0; j < i->width * i->height; ++j ) {
 		i->alpha[j] = 1.0;
 	}
-	memset(i->lookup_resx, 0, (1 + i->width) * sizeof(int) );
+	memset(i->lookup_resx, 0, (1 + i->src_width) * sizeof(int) );
 
 	if ( usecolors ) {
 		memset(i->red,   0, i->width * i->height * sizeof(float));
@@ -356,11 +489,11 @@ void normalize(Image* i) {
 
 	int x, y;
 
-	for ( y=0; y < i->height; ++y ) {
+	for ( y=0; y < i->src_height; ++y ) {
 
 		if ( i->yadds[y] > 1 ) {
 
-			for ( x=0; x < i->width; ++x ) {
+			for ( x=0; x < i->src_width; ++x ) {
 				pixel[x] /= i->yadds[y];
 
 				if ( usecolors ) {
@@ -371,12 +504,12 @@ void normalize(Image* i) {
 			}
 		}
 
-		pixel += i->width;
+		pixel += i->src_width;
 
 		if ( usecolors ) {
-			red   += i->width;
-			green += i->width;
-			blue  += i->width;
+			red   += i->src_width;
+			green += i->src_width;
+			blue  += i->src_width;
 		}
 	}
 }
@@ -397,10 +530,36 @@ void print_progress(float progress) {
 	fflush(stderr);
 }
 
-void print_info_jpeg(const struct jpeg_decompress_struct* jpg) {
+void print_info_jpeg(const struct jpeg_decompress_struct* jpg, const Orientation orientation) {
 	fprintf(stderr, "Source width: %d\n", jpg->output_width);
 	fprintf(stderr, "Source height: %d\n", jpg->output_height);
 	fprintf(stderr, "Source color components: %d\n", jpg->output_components);
+	switch ( orientation ) {
+		case HORIZONTAL:
+			fprintf(stderr, "Orientation: 1 (Horizontal/normal)\n");
+			break;
+		case MIRROR_HORIZONTAL:
+			fprintf(stderr, "Orientation: 2 (Mirror horizontal)\n");
+			break;
+		case ROTATE_180:
+			fprintf(stderr, "Orientation: 3 (Rotate 180)\n");
+			break;
+		case MIRROR_VERTICAL:
+			fprintf(stderr, "Orientation: 4 (Mirror vertical)\n");
+			break;
+		case MIRROR_HORIZONTAL_ROTATE_90:
+			fprintf(stderr, "Orientation: 5 (Mirror horizontal and rotate 90)\n");
+			break;
+		case ROTATE_270:
+			fprintf(stderr, "Orientation: 6 (Rotate 270)\n");
+			break;
+		case MIRROR_HORIZONTAL_ROTATE_270:
+			fprintf(stderr, "Orientation: 7 (Mirror horizontal and rotate 270)\n");
+			break;
+		case ROTATE_90:
+			fprintf(stderr, "Orientation: 8 (Rotate 90)\n");
+			break;
+	}
 	fprintf(stderr, "Output width: %d\n", width);
 	fprintf(stderr, "Output height: %d\n", height);
 	fprintf(stderr, "Output palette (%d chars): '%s'\n", ascii_palette_length, ascii_palette);
@@ -441,6 +600,24 @@ void print_info_png(const png_structp png_ptr, const png_infop info_ptr) {
 	fprintf(stderr, "Output palette (%d chars): '%s'\n", ascii_palette_length, ascii_palette);
 }
 
+void print_info_webp(WebPDecoderConfig* config) {
+	fprintf(stderr, "Source width: %d\n", config->input.width);
+	fprintf(stderr, "Source height: %d\n", config->input.height);
+	if ( config->input.has_alpha ) {
+		fprintf(stderr, "Has alpha channel: Yes\n");
+	} else {
+		fprintf(stderr, "Has alpha channel: No\n");
+	}
+	if ( config->input.has_animation ) {
+		fprintf(stderr, "Has animation: Yes\n");
+	} else {
+		fprintf(stderr, "Has animation: No\n");
+	}
+	fprintf(stderr, "Output width: %d\n", width);
+	fprintf(stderr, "Output height: %d\n", height);
+	fprintf(stderr, "Output palette (%d chars): '%s'\n", ascii_palette_length, ascii_palette);
+}
+
 void process_scanline_jpeg(const struct jpeg_decompress_struct *jpg, const JSAMPLE* scanline, Image* i) {
 	static int lasty = 0;
 	const int y = ROUND( i->resize_y * (float) (jpg->output_scanline-1) );
@@ -449,12 +626,12 @@ void process_scanline_jpeg(const struct jpeg_decompress_struct *jpg, const JSAMP
 
 	float *pixel, *red, *green, *blue, *alpha;
 
-	pixel  = &i->pixel[lasty * i->width];
+	pixel  = &i->pixel[lasty * i->src_width];
 	red = green = blue = NULL;
-	alpha  = &i->alpha[lasty * i->width];
+	alpha  = &i->alpha[lasty * i->src_width];
 
 	if ( usecolors ) {
-		int offset = lasty * i->width;
+		int offset = lasty * i->src_width;
 		red   = &i->red  [offset];
 		green = &i->green[offset];
 		blue  = &i->blue [offset];
@@ -465,7 +642,7 @@ void process_scanline_jpeg(const struct jpeg_decompress_struct *jpg, const JSAMP
 		const int components = jpg->out_color_components;
 
 		int x;
-		for ( x=0; x < i->width; ++x ) {
+		for ( x=0; x < i->src_width; ++x ) {
 			const JSAMPLE *src     = &scanline[i->lookup_resx[x] * jpg->out_color_components];
 			const JSAMPLE *src_end = &scanline[i->lookup_resx[x+1] * jpg->out_color_components];
 
@@ -504,13 +681,13 @@ void process_scanline_jpeg(const struct jpeg_decompress_struct *jpg, const JSAMP
 
 		++i->yadds[lasty++];
 
-		pixel += i->width;
-		alpha += i->width;
+		pixel += i->src_width;
+		alpha += i->src_width;
 
 		if ( usecolors ) {
-			red   += i->width;
-			green += i->width;
-			blue  += i->width;
+			red   += i->src_width;
+			green += i->src_width;
+			blue  += i->src_width;
 		}
 	}
 
@@ -525,12 +702,12 @@ void process_scanline_png(const png_bytep row, const int current_y, const int co
 
 	float *pixel, *red, *green, *blue, *alpha;
 
-	pixel  = &i->pixel[lasty * i->width];
+	pixel  = &i->pixel[lasty * i->src_width];
 	red = green = blue = NULL;
-	alpha = &i->alpha[lasty * i->width];
+	alpha = &i->alpha[lasty * i->src_width];
 
 	if ( usecolors ) {
-		int offset = lasty * i->width;
+		int offset = lasty * i->src_width;
 		red   = &i->red  [offset];
 		green = &i->green[offset];
 		blue  = &i->blue [offset];
@@ -539,7 +716,7 @@ void process_scanline_png(const png_bytep row, const int current_y, const int co
 	while ( lasty <= y ) {
 
 		int x;
-		for ( x=0; x < i->width; ++x ) {
+		for ( x=0; x < i->src_width; ++x ) {
 
 			int adds = 0;
 
@@ -587,13 +764,13 @@ void process_scanline_png(const png_bytep row, const int current_y, const int co
 
 		++i->yadds[lasty++];
 
-		pixel += i->width;
-		alpha += i->width;
+		pixel += i->src_width;
+		alpha += i->src_width;
 
 		if ( usecolors ) {
-			red   += i->width;
-			green += i->width;
-			blue  += i->width;
+			red   += i->src_width;
+			green += i->src_width;
+			blue  += i->src_width;
 		}
 	}
 
@@ -610,15 +787,19 @@ void free_image(Image* i) {
 	if ( i->lookup_resx ) free(i->lookup_resx);
 }
 
-void malloc_image(Image* i) {
+void malloc_image(Image* i, int switch_x_y) {
+	i->orientation = HORIZONTAL;
+	i->switch_x_y = switch_x_y;
 	i->pixel = i->red = i->green = i->blue = i->alpha = NULL;
 	i->yadds = NULL;
 	i->lookup_resx = NULL;
 
 	i->width = width;
 	i->height = height;
+	i->src_width = switch_x_y ? height : width;
+	i->src_height = switch_x_y ? width : height;
 
-	i->yadds = (int*) malloc(height * sizeof(int));
+	i->yadds = (int*) malloc(i->src_height * sizeof(int));
 	i->pixel = (float*) malloc(width*height*sizeof(float));
 	i->alpha = (float*) malloc(width*height*sizeof(float));
 
@@ -629,7 +810,7 @@ void malloc_image(Image* i) {
 	}
 
 	// we allocate one extra pixel for resx because of the src .. src_end stuff in process_scanline_jpeg and the equivalent in for PNG
-	i->lookup_resx = (int*) malloc( (1 + width) * sizeof(int));
+	i->lookup_resx = (int*) malloc( (1 + i->src_width) * sizeof(int));
 
 	if ( !(i->pixel && i->alpha && i->yadds && i->lookup_resx) ||
 	     (usecolors && !(i->red && i->green && i->blue)) )
@@ -644,21 +825,83 @@ void init_image(Image *i, int src_width, int src_height) {
 	int dst_x;
 
 	if ( src_height > 1 )
-		i->resize_y = (float) (i->height - 1) / (float) (src_height - 1);
+		i->resize_y = (float) (i->src_height - 1) / (float) (src_height - 1);
 	else
 		i->resize_y = 1;
-	i->resize_x = (float) (src_width - 1) / (float) (i->width );
+	i->resize_x = (float) (src_width - 1) / (float) (i->src_width );
 
-	for ( dst_x=0; dst_x <= i->width; ++dst_x ) {
+
+	for ( dst_x=0; dst_x <= i->src_width; ++dst_x ) {
 		i->lookup_resx[dst_x] = ROUND( (float) dst_x * i->resize_x );
 	}
 }
 
+
+Orientation get_orientation(FILE *imageFP) {
+	char orientationTag[13] = "Top-left";  // default to horizontal/normal
+
+	size_t size;
+	unsigned char data[1024];
+	ExifData *edata;
+	ExifLoader *loader;
+	loader = exif_loader_new();
+	while ( 1 ) {
+		size = fread(data, 1, sizeof(data), imageFP);
+		if (size <= 0) {
+			break;
+		}
+		if (!exif_loader_write(loader, data, size)) {
+			break;
+		}
+	}
+	rewind(imageFP);
+	edata = exif_loader_get_data(loader);
+	exif_loader_unref(loader);
+	if ( edata ) {
+		ExifEntry *entry = exif_content_get_entry(edata->ifd[EXIF_IFD_0], EXIF_TAG_ORIENTATION);
+		if ( entry ) {
+			exif_entry_get_value(entry, orientationTag, sizeof(orientationTag));
+		}
+		exif_data_unref(edata);
+	}
+
+	if ( strcmp(orientationTag, "Top-left" ) == 0) {
+		return HORIZONTAL;
+	} else if ( strcmp(orientationTag, "Top-right" ) == 0) {
+		return MIRROR_HORIZONTAL;
+	} else if ( strcmp(orientationTag, "Bottom-right" ) == 0) {
+		return ROTATE_180;
+	} else if ( strcmp(orientationTag, "Bottom-left" ) == 0) {
+		return MIRROR_VERTICAL;
+	} else if ( strcmp(orientationTag, "Left-top" ) == 0) {
+		return MIRROR_HORIZONTAL_ROTATE_90;
+	} else if ( strcmp(orientationTag, "Right-top" ) == 0) {
+		return ROTATE_270;
+	} else if ( strcmp(orientationTag, "Right-bottom" ) == 0) {
+		return MIRROR_HORIZONTAL_ROTATE_270;
+	} else if ( strcmp(orientationTag, "Left-bottom" ) == 0) {
+		return ROTATE_90;
+	}
+	return HORIZONTAL;
+}
+
 void decompress_jpeg(FILE *fp, FILE *fout, error_collector *errors) {
 	if ( errors->jpeg_status ) {
 		print_errors(errors);
 		return;
 	}
+
+	Orientation orientation = get_orientation(fp);
+	int switch_x_y = 0;
+	if (
+			orientation == MIRROR_HORIZONTAL_ROTATE_90 ||
+			orientation == ROTATE_270 ||
+			orientation == MIRROR_HORIZONTAL_ROTATE_270 ||
+			orientation == ROTATE_90
+	) {
+		switch_x_y = 1;
+	}
+
 	int row_stride;
 	my_jpeg_error_mgr jerr;
 	struct jpeg_decompress_struct jpg;
@@ -672,7 +915,7 @@ void decompress_jpeg(FILE *fp, FILE *fout, error_collector *errors) {
 		errors->jpeg_status = 1;
 		jpeg_destroy_decompress(&jpg);
 		rewind(fp);
-		decompress_png(fp, fout, errors);
+		decompress_webp(fp, fout, errors);
 		return;
 	}
 	jpeg_create_decompress(&jpg);
@@ -691,15 +934,16 @@ void decompress_jpeg(FILE *fp, FILE *fout, error_collector *errors) {
 
 	buffer = (*jpg.mem->alloc_sarray)((j_common_ptr) &jpg, JPOOL_IMAGE, row_stride, 1);
 
-	aspect_ratio(jpg.output_width, jpg.output_height);
+	aspect_ratio(jpg.output_width, jpg.output_height, switch_x_y);
 
-	if ( verbose ) print_info_jpeg(&jpg);
+	if ( verbose ) print_info_jpeg(&jpg, orientation);
 
 	if ( height != 0 && width != 0 ) {
-		malloc_image(&image);
+		malloc_image(&image, switch_x_y);
 		clear(&image);
 
 		init_image(&image, jpg.output_width, jpg.output_height);
+		image.orientation = orientation;
 
 		while ( jpg.output_scanline < jpg.output_height ) {
 			jpeg_read_scanlines(&jpg, buffer, 1);
@@ -752,7 +996,7 @@ void decompress_png(FILE *fp, FILE *fout, error_collector *errors) {
 		errors->png_status = 1;
 		png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
 		rewind(fp);
-		decompress_jpeg(fp, fout, errors);
+		decompress_webp(fp, fout, errors);
 		return;
 	}
 	png_init_io(png_ptr, fp);
@@ -762,12 +1006,12 @@ void decompress_png(FILE *fp, FILE *fout, error_collector *errors) {
 	int png_width = png_get_image_width(png_ptr, info_ptr);
 	int png_height = png_get_image_height(png_ptr, info_ptr);
 
-	aspect_ratio(png_width, png_height);
+	aspect_ratio(png_width, png_height, 0);
 
 	if ( verbose ) print_info_png(png_ptr, info_ptr);
 
 	if ( height != 0 && width != 0 ) {
-		malloc_image(&image);
+		malloc_image(&image, 0);
 		clear(&image);
 
 
@@ -832,6 +1076,130 @@ void decompress_png(FILE *fp, FILE *fout, error_collector *errors) {
 	png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
 }
 
+webp_data* get_webp_data(FILE *fp) {
+	webp_data* data_struct = malloc(sizeof(webp_data));
+	uint8_t buffer[1024];
+	size_t bytes_read;
+	size_t data_size = 0;
+	size_t data_allocated_size = 16384;
+	uint8_t* data = malloc(data_allocated_size);
+	
+	while ( 1 ) {
+		bytes_read = fread(buffer, 1, sizeof(buffer), fp);
+		if ( bytes_read <= 0 ) {
+			break;
+		}
+		size_t new_data_size = data_size + bytes_read;
+		if ( new_data_size > data_allocated_size ) {
+			data_allocated_size *= 2;
+			data = realloc(data, data_allocated_size);
+		}
+		uint8_t* dest = data + data_size;
+		memcpy(dest, buffer, bytes_read);
+		data_size = new_data_size;
+	}
+
+	data_struct->data = data;
+	data_struct->size = data_size;
+
+	return data_struct;
+}
+
+void free_webp_data(webp_data* data) {
+	free(data->data);
+	free(data);
+}
+
+void decompress_webp(FILE *fp, FILE *fout, error_collector *errors) {
+	if ( errors->webp_status ) {
+		print_errors(errors);
+		return;
+	}
+
+	Orientation orientation = get_orientation(fp);
+	int switch_x_y = 0;
+	if (
+			orientation == MIRROR_HORIZONTAL_ROTATE_90 ||
+			orientation == ROTATE_270 ||
+			orientation == MIRROR_HORIZONTAL_ROTATE_270 ||
+			orientation == ROTATE_90
+	) {
+		switch_x_y = 1;
+	}
+
+	Image image;
+
+	WebPDecoderConfig config;
+	WebPInitDecoderConfig(&config);
+
+	webp_data* img_data = get_webp_data(fp);
+
+	if ( WebPGetFeatures(img_data->data, img_data->size, &config.input) != VP8_STATUS_OK ) {
+		errors->webp_error_msg = "Unable to determine WebP features, possibly not a WebP";
+		errors->webp_status = 1;
+		free_webp_data(img_data);
+		rewind(fp);
+		decompress_png(fp, fout, errors);
+		return;
+	}
+
+	aspect_ratio(config.input.width, config.input.height, switch_x_y);
+	
+	if ( verbose ) print_info_webp(&config);
+
+	if ( height != 0 && width != 0 ) {
+		malloc_image(&image, switch_x_y);
+		clear(&image);
+
+		init_image(&image, image.src_width, image.src_height);
+		image.orientation = orientation;
+		if ( image.src_width != config.input.width || image.src_height != config.input.height ) {
+			// scale the image using the webp library instead of by jp2a
+			// this should provide smoother and faster scaling
+			config.options.use_scaling = 1;
+			config.options.scaled_width = image.src_width;
+			config.options.scaled_height = image.src_height;
+		}
+		config.output.colorspace = MODE_RGBA;
+		image.resize_x = 1.0f;
+		image.resize_y = 1.0f;
+
+		if ( WebPDecode(img_data->data, img_data->size, &config) != VP8_STATUS_OK) {
+
+			errors->webp_error_msg = "Error decoding WebP image";
+			errors->webp_status = 1;
+			free_image(&image);
+			free_webp_data(img_data);
+			rewind(fp);
+			decompress_png(fp, fout, errors);
+			return;
+		}
+
+		WebPRGBABuffer* u = (WebPRGBABuffer*) &config.output.u;
+		uint8_t* rgba = u->rgba;
+
+		for ( size_t i = 0; i < image.width * image.height; i++ ) {
+			if ( usecolors ) {
+				image.red[i] = rgba[i * 4] / 255.0f;
+				image.green[i] = rgba[i * 4 + 1] / 255.0f;
+				image.blue[i] = rgba[i * 4 + 2] / 255.0f;
+			}
+			image.pixel[i] = RED[rgba[i * 4]] + GREEN[rgba[i * 4 + 1]] + BLUE[rgba[i * 4 + 2]];
+			image.alpha[i] = rgba[i * 4 + 3] / 255.0f;
+		}
+		for ( size_t i = 0; i < image.src_height; i++ ) {
+			image.yadds[i] = 1;
+		}
+
+		print_image(&image, fout);
+
+		free_image(&image);
+		WebPFreeDecBuffer(&config.output);
+	}
+
+	free_webp_data(img_data);
+}
+
 void print_errors(error_collector *errors) {
 	if ( errors->jpeg_status ) {
 		my_jpeg_error_mgr *jerr = errors->jpeg_error;
@@ -842,4 +1210,7 @@ void print_errors(error_collector *errors) {
 	if ( errors->png_status ) {
 		fprintf(stderr, "%s\n", errors->png_error_msg);
 	}
+	if ( errors->webp_status ) {
+		fprintf(stderr, "%s\n", errors->webp_error_msg);
+	}
 }
