@@ -83,6 +83,7 @@ int main(int argc, char** argv) {
 
 		errors.jpeg_status = 0;
 		errors.png_status = 0;
+		errors.webp_status = 0;
 
 		// read from stdin
 		if ( argv[n][0]=='-' && !argv[n][1] ) {
@@ -110,7 +111,7 @@ int main(int argc, char** argv) {
 #endif
 			}
 
-			if ( errors.jpeg_status && errors.png_status )
+			if ( errors.jpeg_status && errors.png_status && errors.webp_status )
 				retval = 1;
 			continue;
 		}
@@ -142,6 +143,8 @@ int main(int argc, char** argv) {
 					int urllen = strlen(argv[n]);
 					if ( urllen > 4 && strcmp(".png", argv[n] + (urllen - 4)) == 0 )
 						decompress_png(buffer_f, fout, &errors);
+					else if ( urllen > 5 && strcmp(".webp", argv[n] + (urllen - 5)) == 0 )
+						decompress_webp(buffer_f, fout, &errors);
 					else
 						decompress_jpeg(buffer_f, fout, &errors);
 					fclose(buffer_f);
@@ -153,7 +156,7 @@ int main(int argc, char** argv) {
 			fclose(fr);
 			close(fd);
 			
-			if ( errors.jpeg_status && errors.png_status )
+			if ( errors.jpeg_status && errors.png_status && errors.webp_status )
 				retval = 1;
 			continue;
 		}
@@ -167,11 +170,13 @@ int main(int argc, char** argv) {
 			int namelen = strlen(argv[n]);
 			if ( namelen > 4 && strcmp(".png", argv[n] + (namelen - 4)) == 0 )
 				decompress_png(fp, fout, &errors);
+			else if ( namelen > 5 && strcmp(".webp", argv[n] + (namelen - 5)) == 0 )
+				decompress_webp(fp, fout, &errors);
 			else
 				decompress_jpeg(fp, fout, &errors);
 			fclose(fp);
 
-			if ( errors.jpeg_status && errors.png_status )
+			if ( errors.jpeg_status && errors.png_status && errors.webp_status )
 				retval = 1;
 			continue;
 
