@@ -10,6 +10,9 @@ JP=../src/jp2a
 UNAME=`uname -a`
 UPDATE=no
 
+## FOR LOCAL DEPENDENT INPUTS LIKE --edge-threshold=0.3 TO BE INTERPRETED CORRECTLY
+export LC_ALL=C
+
 if test "`echo ${UNAME} | cut -c1-6`" == "CYGWIN" ; then
 	JP=../src/jp2a.exe
 fi
@@ -80,6 +83,8 @@ echo " completely broken."
 echo "-------------------------------------------------------------"
 echo ""
 
+export COLORTERM=truecolor
+
 test_jp2a "width" "--width=78 jp2a.jpg" normal.txt
 test_jp2a "border, width" "-b --width=78 jp2a.jpg" normal-b.txt
 test_jp2a "size" "--size=160x49 jp2a.jpg" 160x49.txt
@@ -152,9 +157,24 @@ test_jp2a "big size" "--size=2000x2000 dalsnuten-640x480-gray-low.jpg jp2a.jpg |
 test_jp2a "PNG invert" "--invert --color --size=144x36 jp2a-colorful.png" jp2a-png-invert.txt
 test_jp2a "PNG colormap standard input" "2>/dev/null ; cat jp2a-colormap.png | ${JP} - --size=144x36" jp2a-png-colormap.txt
 test_jp2a "PNG interlaced 16-bit" "--size=300x150 --color jp2a-interlaced-16-bit.png" jp2a-png-interlaced-16-bit.txt
+test_jp2a "WebP" "--width=78 jp2a.webp" jp2a-webp.txt
+test_jp2a "WebP scale up" "--width=120 jp2a.webp" jp2a-webp-scale-up.txt
 test_jp2a "size, curl download" "--size=454x207 http://jp2a.sourceforge.net/jp2a.jpg" normal-curl.txt
 test_jp2a "size, curl download" "--size=454x207 http://jp2a.sf.net/jp2a.jpg" normal-curl.txt
 test_jp2a "very long jpg" "long.jpg" no-output.txt
 test_jp2a "very long png" "long.png" no-output.txt
+test_jp2a "Exif orientation 2" "--width=78 jp2a-mirror-horizontal.jpg" jp2a-mirror-horizontal.txt
+test_jp2a "Exif orientation 3" "--width=78 jp2a-rotate-180.jpg" jp2a-rotate-180.txt
+test_jp2a "Exif orientation 4" "--width=78 jp2a-mirror-vertical.jpg" jp2a-mirror-vertical.txt
+test_jp2a "Exif orientation 5" "--width=78 jp2a-horizontal-270.jpg" jp2a-horizontal-270.txt
+test_jp2a "Exif orientation 6" "--width=78 jp2a-rotate-90.jpg" jp2a-rotate-90.txt
+test_jp2a "Exif orientation 7" "--width=78 jp2a-horizontal-90.jpg" jp2a-horizontal-90.txt
+test_jp2a "Exif orientation 8" "--width=78 jp2a-rotate-270.jpg" jp2a-rotate-270.txt
+test_jp2a "Exif orientation 5, color" "--width=78 --color --color-depth=24 jp2a-colorful-horizontal-270.jpg" jp2a-colorful-horizontal-270.txt
+test_jp2a "Exif orientation 2, flipx" "--width=78 --flipx jp2a-mirror-horizontal.jpg" jp2a-mirror-horizontal-flipx.txt
+test_jp2a "Exif orientation 6, flipy" "--width=78 --flipy jp2a-rotate-90.jpg" jp2a-rotate-90-flipy.txt
+test_jp2a "edges" "--width=128 --edge-threshold=0.3 jp2a.jpg" edges.txt
+test_jp2a "edges only" "--width=128 --edge-threshold=0.3 --edges-only jp2a.jpg" edges-only.txt
+test_jp2a "edges only, invert" "--width=128 --edge-threshold=0.3 --edges-only --invert jp2a.jpg" edges-only-invert.txt
 
 test_results
